﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
using log4net;

namespace SGPM.CrossCutting.Logging
{
    public class Tracer
    {
        #region Variables locales
        private static ILog _logger;
        private static Tracer _instance;
        private static readonly object padlock = new object();
        private bool _initialized;
        #endregion

        #region Métodos/Propiedades públicos

        #region Constructor
        private Tracer()
        {
            //Declaración privada del constructor: constructor de instancia. 
        }
        #endregion

        public void Initialize(string logger)
        {
            log4net.Config.XmlConfigurator.Configure(new System.IO.FileInfo(ConfigurationManager.AppSettings["Log4NetPath"]));
            _logger = LogManager.GetLogger(logger);
            _initialized = true;
        }

        private void checkInitialized()
        {
            if (!_initialized)
                throw new Exception("El componente Log4Net no está inicializado");
        }

        public void Debug(object message, Exception exception)
        {
            checkInitialized();
            _logger.Debug(message, exception);
        }

        public void Debug(object message)
        {
            checkInitialized();
            _logger.Debug(message);
        }

        public void Error(object message, Exception exception)
        {
            checkInitialized();
            _logger.Error(message, exception);
        }
        public void Error(object message)
        {
            checkInitialized();
            _logger.Error(message);
        }

        public void Error(Exception exception)
        {
            checkInitialized();
            _logger.Error(exception);
        }

        public void Info(object message)
        {
            checkInitialized();
            _logger.Info(message);
        }

        public void Warn(object message)
        {
            checkInitialized();
            _logger.Warn(message);
        }

        public void Fatal(object message)
        {
            checkInitialized();
            _logger.Fatal(message);
        }

        #endregion

        #region propiedades

        public static Tracer Logger
        {
            get
            {
                lock (padlock)
                {
                    if (_instance == null)
                    {
                        _instance = new Tracer();
                    }
                    return _instance;
                }
            }
        }
        #endregion

    }
}
