﻿using System;
using System.Linq;
using System.Windows;
using SGPM.PhonePER.ModeloDatos;
using SGPM.PhonePER.ProxyWSPhonePER;
using Microsoft.Phone.Controls;

namespace SGPM.PhonePER
{
    public partial class CambioPassPage : PhoneApplicationPage
    {

        #region Variables miembro

        // Instancia del servicio web
        WSPhonePERClient _ws = new WSPhonePERClient();
        private Perito _Perito; // Perito que ha abierto la sesión

        #endregion

        #region Constructor

        public CambioPassPage()
        {

            // Se asigna El manejador de eventos para el cambio de la password
            _ws.CambiarPasswordCompleted  +=
                new EventHandler<CambiarPasswordCompletedEventArgs>(CambioPassword_Completed);

            using (SGPMDataContext _DataContext = new SGPMDataContext())
            {
                // Establecemos el perito actual
                _Perito = _DataContext.Peritos.First();

                // Asignamos el contexto de datos para el enlace con los controles
                this.DataContext = _Perito;
            }

            InitializeComponent();
        }

        #endregion

        #region Manejadores de eventos para el cambio de la password

        private void CambioPassword_Completed(object sender, CambiarPasswordCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show("No se ha podido realizar el cambio de password.", "Error", MessageBoxButton.OK);
            }
            else
            {
                if (e.Result)
                {
                    using (SGPMDataContext _DataContext = new SGPMDataContext())
                    {
                        _Perito.Password = this.tbxPassword1.Password;
                        _DataContext.SubmitChanges();
                    }
                    MessageBox.Show("Se ha realizado el cambio de password.", "Cambio de password", MessageBoxButton.OK);

                    // Volvemos a la página principal
                    this.NavigationService.GoBack();
                }
                else
                {
                    MessageBox.Show("El servidor ha rechazado el cambio de password. Compruebe que la password actual es correcta.", "Error en el cambio de Password", MessageBoxButton.OK);
                }
            }
        }

        #endregion

        #region Manejadores de eventos de la botonera

        private void btnChangePassword_Click(object sender, EventArgs e)
        {
            // Comprobación de errores
            if (this.tbxActualPassword.Password != _Perito.Password)
            {
                MessageBox.Show("La password actual proporcionada es incorrecta.", "Error", MessageBoxButton.OK);
            }
            else if (string.IsNullOrWhiteSpace(this.tbxPassword1.Password) ||
                     this.tbxPassword1.Password.Length < 4)
            {
                MessageBox.Show("La password que se intenta establecer no cumple con los requerimientos mínimos de seguridad.", "Error", MessageBoxButton.OK);
            }
            else if (this.tbxPassword1.Password != this.tbxPassword2.Password)
            {
                MessageBox.Show("La confirmación de la password no coincide con la contraseña que se intenta establecer.", "Error", MessageBoxButton.OK);
            }
            else if (this.tbxActualPassword.Password == this.tbxPassword1.Password)
            {
                MessageBox.Show("La password que se intenta establecer es idéntica a la actual.", "Error", MessageBoxButton.OK);
            }
            else
            {
                // Llamamos al método del servicio web para cambiar la contraseña
                _ws.CambiarPasswordAsync (_Perito, this.tbxPassword1.Password);
            }
        }

        /// <summary>
        /// Manejador de evento del botón Cancelar
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            tbxActualPassword.Password = "";
            tbxPassword1.Password = "";
            tbxPassword2.Password = "";
        }

        #endregion

    }
}