﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Data.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Data;
using System.Windows.Navigation;
using System.Windows.Threading;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using System.IO.IsolatedStorage;
using System.IO;
using SGPM.PhonePER.ModeloDatos;
using SGPM.PhonePER.Enums;
using SGPM.PhonePER.ProxyWSPhonePER;

using Microsoft.Phone.Notification;
using System.Net.NetworkInformation;
using System.Text;
using System.Windows.Controls.Primitives;

namespace SGPM.PhonePER
{
    /// <summary>
    /// Página principal de la aplicación. Muestra el listado de órdenes asignadas al perito pendientes de valorar
    /// </summary>
    public partial class ListOrdenesPage : PhoneApplicationPage
    {

        #region Variables miembro
        
        // Instancia del servicio web
        WSPhonePERClient _ws = new WSPhonePERClient();
        // Perito que ha abierto la sesión
        private Perito _Perito;
        // Transferencias pendientes
        private volatile int _Transferencias; 
        // Colección de órdenes de trabajo
        private ObservableCollection<OrdenTrabajo> _OrdenesTrabajo = new ObservableCollection<OrdenTrabajo>();

        //Popup de ayuda
        Popup _help = new Popup();

        #endregion

        #region Constructor

        public ListOrdenesPage()
        {
            InitializeComponent();

            // Establecer el contexto de datos del control ListBox control 
            DataContext = _OrdenesTrabajo;
            this.Loaded += new RoutedEventHandler(ListOrdenesPage_Loaded);

            // Se asignan los manejadores de eventos para indicar al servicio web la URI de suscripción al
            // Servicio de notificaciones de Microsoft
            _ws.SetSubscriptionURICompleted +=
                new EventHandler<SetSubscriptionURICompletedEventArgs>(LeerURI_Completed);

            // Se asignan los manejadores de eventos para la descarga y transferencia de órdenes de trabajo
            _ws.ListOrdenesCompleted +=
                new EventHandler<ListOrdenesCompletedEventArgs>(DescargaOrdenes_Completed);
            _ws.UpdateOrdenCompleted +=
                new EventHandler<UpdateOrdenCompletedEventArgs>(TransferirOrden_Completed);

            // Se asigna el manejador de eventos para el método que se dispara cuando hay cambios en la conexión
            NetworkChange.NetworkAddressChanged += 
                new NetworkAddressChangedEventHandler(NetworkChange_NetworkAddressChanged);

        }

        #endregion

        #region Manejador de eventos de la página

        private void ListOrdenesPage_Loaded(object sender, RoutedEventArgs e)
        {
            if (!App.Autenticado.HasValue || App.Autenticado == false)
            {
                // Se redirige a la autenticación
                this.NavigationService.Navigate(new Uri("/UserPassPage.xaml", UriKind.Relative));
            }
            else
            {
                // Se quita el manejador de eventos de la carga de la página, para que no se ejecute varias veces
                this.Loaded -= new RoutedEventHandler(ListOrdenesPage_Loaded);

                // Mostramos el menú
                this.ApplicationBar.IsVisible = true;
                using (SGPMDataContext _DataContext = new SGPMDataContext())
                {
                    // Establecemos el perito actual
                    _Perito = _DataContext.Peritos.First();
                }

                // Abrimos el canal push para recibir notificaciones de actualización en órdenes
                this.AbrirCanalPush();

                // Lanzamos la descarga de las órdenes de trabajo
                this.DescargarOrdenes();

                // Lanzamos la transferencia al servidor de las órdenes de trabajo valoradas
                this.Transferir();

            }
        }

        #endregion

        #region Métodos y eventos de la botonera

        private void btnCambioPass_Click(object sender, EventArgs e)
        {
            // Abrimos la página para el cambio de password
            this.NavigationService.Navigate(new Uri("/CambioPassPage.xaml", UriKind.Relative));
        }

        private void btnTransferir_Click(object sender, EventArgs e)
        {
            this.Transferir();
        }

        private void btnDescargar_Click(object sender, EventArgs e)
        {
            this.DescargarOrdenes();
        }

        private void EnableMenu()
        {
            Dispatcher.BeginInvoke(() =>
            {
                // Habilitamos cada botón del menú
                foreach (ApplicationBarIconButton button in this.ApplicationBar.Buttons)
                {
                    button.IsEnabled = true;
                }
            });
        }

        private void DisableMenu()
        {

            Dispatcher.BeginInvoke(()=>
            {
                // Deshabilitamos cada botón del menú
                foreach (ApplicationBarIconButton button in this.ApplicationBar.Buttons)
                {
                    button.IsEnabled = false;
                }
            });
        }

        /// <summary>
        /// Manejador de evento para el botón de ayuda
        /// Crea una ventana popup para mostrar el contenido de la ayuda
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void btnAyuda_Click(object sender, EventArgs e)
        {
            //Stack panel cn el fondo negro
            StackPanel panelHelp = new StackPanel();
            panelHelp.Background = new SolidColorBrush(Colors.Black);
            panelHelp.Width = 300;
            panelHelp.Height = 400;

            //Crea un borde blanco
            Border border = new Border();
            border.BorderBrush = new SolidColorBrush(Colors.White);
            border.BorderThickness = new Thickness(7.0);

            //Boton de cerrar para salir del popup
            Button close = new Button();
            close.Content = "Cerrar";
            close.Margin = new Thickness(5.0);
            close.Click += new RoutedEventHandler(close_Click);

            //Crea el texto de la ayuda
            TextBlock textblockHelp = new TextBlock();
            textblockHelp.FontSize = 24;
            textblockHelp.Foreground = new SolidColorBrush(Colors.White);
            textblockHelp.TextWrapping = TextWrapping.Wrap;
            textblockHelp.Text = "Mantenga seleccionada la orden de trabajo que desee valorar.\n" +
                "Esto mostrará el formulario de valoración de la orden.\n";
            textblockHelp.Margin = new Thickness(5.0);

            //Añade los controles al stack panel
            panelHelp.Children.Add(textblockHelp);
            panelHelp.Children.Add(close);
            border.Child = panelHelp;

            // Inicializa la propiedad Child del popup
            _help.Child = border;

            // Localización en la pantalla de la ventana popup
            _help.VerticalOffset = 200;
            _help.HorizontalOffset = 85;

            // Se abre la ventana
            _help.IsOpen = true;
        }

        /// <summary>
        /// Manejador de evento para el botón cerrar de la ventana de ayuda
        /// Cierra el popup
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void close_Click(object sender, RoutedEventArgs e)
        {
            _help.IsOpen = false;
        }
        #endregion 

        #region Métodos y eventos de transferencia y descarga de órdenes

        /// <summary>
        /// Envía las órdenes de trabajo ya valoradas por el perito al servidor.
        /// También adjunta la foto cuando corresponde
        /// </summary>
        private void Transferir()
        {

            // Comprobamos que tenemos conectividad
            if (!Microsoft.Phone.Net.NetworkInformation.NetworkInterface.GetIsNetworkAvailable())
            {
                //No hay conectividad, salimos
                return;
            }

            // Desabilitamos el menú
            this.DisableMenu();

            using (SGPMDataContext _DataContext = new SGPMDataContext())
            {
                // Obtenemos las órdenes de trabajo que se  han completado
                var ordenes = _DataContext.OrdenesTrabajo.Where(o => o.Aceptada > (int)ValoracionOrden.SinValorar).ToArray();

                if (ordenes.Length > 0)
                {
                    // Actualizamos el contador de transferencias
                    _Transferencias = ordenes.Length;

                    // Se procesa cada orden
                    foreach (var orden in ordenes)
                    {
                        // Junto con los datos se adjunta la foto
                        string fileName = string.Format("TempJPEG_{0}.jpg", orden.IdOrdenTrabajo);
                        _ws.UpdateOrdenAsync(_Perito, orden, GetFotoBytes(fileName), orden);
                    }
                }
                else
                {
                    // No hay actualizaciones pendientes, así que habilitamos de nuevo el menú
                    this.EnableMenu();
                    _Transferencias = 0;

                }
            }
        }

        private void DescargarOrdenes()
        {
            // Comprobamos que tenemos conectividad
            if (!Microsoft.Phone.Net.NetworkInformation.NetworkInterface.GetIsNetworkAvailable())
            {
                //No hay conectividad, salimos
                return;
            }

            // Desabilitamos el menú
            this.DisableMenu();

            // Llamamos al método del servicio web para descargar las órdenes
            _ws.ListOrdenesAsync(_Perito);
        }

        private void LeerURI_Completed(object sender, SetSubscriptionURICompletedEventArgs e)
        {
            return;
        }

        /// <summary>
        /// Método que se dispara cuando se finaliza la transferencia de una orden al servidor. 
        /// Se encarga de eliminar la orden de la base de datos local y la foto anexa, si la tuviese.
        /// <param name="sender"></param>
        /// <param name="e">Nos permite acceder a la orden de trabajo transferida</param>
        /// </summary>
        private void TransferirOrden_Completed(object sender, UpdateOrdenCompletedEventArgs e)
        {
            // Decrementamos el contador de transferencias 
            _Transferencias--;

            if (e.Result)
            {
                // La actualización ha tenido éxito, así que ya podemos eliminar la órden de trabajo de la base de datos local
                var ordenTrabajo = e.UserState as OrdenTrabajo;
                _OrdenesTrabajo.Remove(ordenTrabajo);

                using (SGPMDataContext _DataContext = new SGPMDataContext())
                {
                    //Recupera la orden de trabajo
                    OrdenTrabajo existingObject = _DataContext.OrdenesTrabajo.Where(x => x.IdOrdenTrabajo == ordenTrabajo.IdOrdenTrabajo).FirstOrDefault();

                    _DataContext.OrdenesTrabajo.DeleteOnSubmit(existingObject);
                    _DataContext.SubmitChanges();

                    //Eliminamos la foto (en caso que exista)
                    string fileName = string.Format("TempJPEG_{0}.jpg", existingObject.IdOrdenTrabajo);
                    DeleteFile(fileName);
                }
            }

            if (_Transferencias == 0)
            {
                // Habilitamos los botones menú
                this.EnableMenu();
            }
        }

        /// <summary>
        /// Método que se dispara cuando se finaliza la descarga de las órdenes desde el servidor. 
        /// Se encarga de insertar las órdenes a la base de datos local y eliminar aquellas que se hayan
        /// desasignado al perito.
        /// <param name="sender"></param>
        /// <param name="e">Nos permite acceder a la orden de trabajo transferida</param>
        /// </summary>
        private void DescargaOrdenes_Completed(object sender, ListOrdenesCompletedEventArgs e)
        {
            var ordenes = e.Result;

                // Comprobamos si hay órdenes de trabajo pendientes
                if (ordenes != null &&
                    ordenes.Count > 0)
                {
                    try
                    {
                        using (SGPMDataContext _DataContext = new SGPMDataContext())
                        {
                            // Procesamos cada una de ellas
                            foreach (var ordenTrabajo in ordenes)
                            {
                                // Comprobamos que no exista en la base de datos
                                if (!_DataContext.OrdenesTrabajo.Any(o => o.IdOrdenTrabajo == ordenTrabajo.IdOrdenTrabajo))
                                {
                                    // Si no existe en la base de datos local la añadimos
                                    _DataContext.OrdenesTrabajo.InsertOnSubmit(ordenTrabajo);
                                }
                            }

                            // Eliminamos las que se hayan desasignado al perito
                            foreach (var ordenTrabajo in _DataContext.OrdenesTrabajo)
                            {
                                // Comprobamos que no exista en la lista de órdenes descargadas
                                if (!ordenes.Any(o => o.IdOrdenTrabajo == ordenTrabajo.IdOrdenTrabajo))
                                {
                                    // Si no existe en las ordenes descargadas la eliminamos de la base de datos local
                                    _DataContext.OrdenesTrabajo.DeleteOnSubmit(ordenTrabajo);
                                }
                            }

                            // Guardamos los cambios en la base de datos local
                            _DataContext.SubmitChanges();
                        } // using _DataContext
                    }
                    catch (Exception ex)
                    {
                        System.Diagnostics.Debug.WriteLine(ex);
                        MessageBox.Show("Se ha producido un error en la descarga de órdenes de trabajo\nError: ." + ex.Message, "Error", MessageBoxButton.OK);
                    }

                    // Se recarga el ListBox con las ordenes pendientes de valoración (Sin valorar) que pertenecen la perito logeado
                    // Usamos Dispatcher ya que puede ejecutarse por una notificación push, de forma que el hilo que lo ejecuta
                    // es distinto al de la página
                    Dispatcher.BeginInvoke(() =>
                    {
                        try 
                        {
                            using (SGPMDataContext _DataContext = new SGPMDataContext())
                            {
                                _OrdenesTrabajo.Clear();
                                _DataContext.OrdenesTrabajo.Where(o =>
                                    o.Aceptada == (int)ValoracionOrden.SinValorar).ToList().ForEach(o => _OrdenesTrabajo.Add(o));
                                var ordenTrabajo = this.Resources["OrdenTrabajo"] as CollectionViewSource;
                                ordenTrabajo.View.Refresh();

                            }
                            
                        }
                        catch (Exception ex)
                        {
                            System.Diagnostics.Debug.WriteLine(ex);
                            MessageBox.Show("Se ha producido un error en la descarga de órdenes de trabajo\nError: ." + ex.Message, "Error", MessageBoxButton.OK);
                        }
                    });

                }

            // Habilitamos los botones menú
            this.EnableMenu();

        }

        #endregion

        #region Métodos de gestión de ficheros

        /// <summary>
        /// Retorna un array de bytes con el contenido de un fichero. En este caso será la foto de una orden
        /// <param name="fileName">Nombre del fichero</param>
        /// <returns>Un array de bytes con el fichero</returns>
        /// </summary>
        private byte[] GetFotoBytes(string fileName)
        {

            using (var appStorage =
                IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (appStorage.FileExists(fileName))
                {
                    IsolatedStorageFileStream isoStream =
                        appStorage.OpenFile(fileName,
                            System.IO.FileMode.Open);

                    byte[] buffer = new byte[isoStream.Length];
                    isoStream.Read(buffer, 0, (int)isoStream.Length);
                    isoStream.Close();

                    return buffer;
                }
                else
                {
                    return null;
                }
            }
        }

        /// <summary>
        /// Elimina un fichero del almacenamiento local, en caso que exista.
        /// <param name="fileName">Nombre del fichero</param>
        /// </summary>
        private void DeleteFile(string fileName)
        {

            using (var appStorage =
                IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (appStorage.FileExists(fileName))
                {
                    appStorage.DeleteFile(fileName);
                    
                }
            }
        }
        #endregion

        #region Eventos del listbox (selección de órdenes)

        private void OnOrdenTrabajoHold(object sender, Microsoft.Phone.Controls.GestureEventArgs e)
        {
            // Obtenemos la órden de trabajo seleccionada
            var ordenTrabajo = (sender as StackPanel).DataContext as OrdenTrabajo;

            string parametros = string.Format("?IdOrdenTrabajo={0}", ordenTrabajo.IdOrdenTrabajo);

            // Abrimos la página para actualizar la orden de trabajo
            this.NavigationService.Navigate(new Uri(string.Format("/OrdenTrabajoPage.xaml{0}", parametros), 
                UriKind.Relative));
        }

        private void OnOrdenTrabajoFiltering(object sender, FilterEventArgs e)
        {
            // Filtramos las órdenes de trabajo en el control visual (sólo necesitamos las no procesadas)
            e.Accepted = false;
            var ordenTrabajo = e.Item as OrdenTrabajo;
            if (ordenTrabajo != null)
            {
                // Filtramos por las no procesadas (no tiene valor 
                e.Accepted = (ordenTrabajo.Aceptada == (int)ValoracionOrden.SinValorar);
            }
        }

        #endregion

        #region Eventos de navegación de la página

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            // Actualizamos las órdenes de trabajo cada vez que se vuelva a la página principal
            var ordenTrabajo = this.Resources["OrdenTrabajo"] as CollectionViewSource;
            if (ordenTrabajo != null &&
                ordenTrabajo.View != null)
            {
                // Hemos retornado desde el detalle de una orden
                using (SGPMDataContext _DataContext = new SGPMDataContext())
                {
                    // Se refresca el listado de órdenes
                    _OrdenesTrabajo.Clear();
                    // Se muestran las órdenes de trabajo pendientes de valorar
                    _DataContext.OrdenesTrabajo.Where(o => 
                        o.Aceptada == (int)ValoracionOrden.SinValorar).ToList().ForEach(o => _OrdenesTrabajo.Add(o));
                    ordenTrabajo.View.Refresh();

                    // Si hay órdenes valoradas pendientes de transferir, invocamos la transferencia
                    if (_DataContext.OrdenesTrabajo.Where(o => o.Aceptada != (int)ValoracionOrden.SinValorar).Count() > 0)
                    {
                        this.Transferir();
                    }

                }

            }
        }

        #endregion

        #region Eventos Push

        /// <summary>
        /// Método que permite abrir el canal push para recepcionar las notificaciones Toast
        /// </summary>
        private void AbrirCanalPush()
        {
            // Gestión de las notificaciones Toast
            HttpNotificationChannel CanalPush;

            string NombreCanal = "CanalSGPM";

            CanalPush = HttpNotificationChannel.Find(NombreCanal);

            // Comprobamos si el canal existe
            if (CanalPush == null)
            {
                // No existe, lo creamos
                CanalPush = new HttpNotificationChannel(NombreCanal);

                // Asignamos los manejadores de eventos
                CanalPush.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(PushChannel_ChannelUriUpdated);
                CanalPush.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(PushChannel_ErrorOccurred);
                CanalPush.ShellToastNotificationReceived += new EventHandler<NotificationEventArgs>(PushChannel_ShellToastNotificationReceived);
                CanalPush.Open();

                // Dejamos el canal a la espera de recibir notificaciones Toast
                CanalPush.BindToShellToast();

            }
            else
            {
                // Asignamos los manejadores de eventos
                CanalPush.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(PushChannel_ChannelUriUpdated);
                CanalPush.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(PushChannel_ErrorOccurred);
                CanalPush.ShellToastNotificationReceived += new EventHandler<NotificationEventArgs>(PushChannel_ShellToastNotificationReceived);

                //Mostramos la Uri del canal en la página
                //Sólo usado a efectos de depuración
                //System.Diagnostics.Debug.WriteLine(CanalPush.ChannelUri.ToString());
            }
        }

        /// <summary>
        /// Este manejador de evento se dispara cuando la Uri asignada al dispositivo cambia (o es la primera vez que se le asigna). Hay que 
        /// notificarlo al servicio web para que relacione la Uri con el perito y su dispositivo, y de esta forma cada perito reciba
        /// sólo las órdenes que le interesen.
        /// </summary>
        void PushChannel_ChannelUriUpdated(object sender, NotificationChannelUriEventArgs e)
        {
            // Notificamos al servicio web la URI asignada al dispositivo móvil del perito
            _ws.SetSubscriptionURIAsync(_Perito, e.ChannelUri.ToString());
        }

        /// <summary>
        /// Manejador de eventos cuando se produce un error en la notificación push
        /// </summary>
        void PushChannel_ErrorOccurred(object sender, NotificationChannelErrorEventArgs e)
        {
            // Usamos el dispacher de la página porque los eventos de HttpNotificationChannel
            // se ejecutan en un hilo diferente al de la aplicación
            Dispatcher.BeginInvoke((Action)delegate()
            {
                string error = string.Format("Se ha producido un problema {0} al intentar recibir notificaciones.\nInfo. Técnica: {1} ({2}) {3}",  
                    e.ErrorType, e.Message, e.ErrorCode, e.ErrorAdditionalData);
                MessageBox.Show(error);
            });
        }

        /// <summary>
        /// Este manejador de evento gestiona la recepción de notificaciones Toast
        /// </summary>
        void PushChannel_ShellToastNotificationReceived(object sender, NotificationEventArgs e)
        {
            // Hay actualizaciones en las órdenes asignadas al perito

            // Descomentar para depuración de notificaciones Toast
            StringBuilder mensaje = new StringBuilder();
            mensaje.AppendFormat("PhonePER: {0}", e.Collection["wp:Text1"]);
            Dispatcher.BeginInvoke(() => MessageBox.Show(mensaje.ToString()));

            // Lanzamos la actualización de órdenes
            this.DescargarOrdenes();

        }
        #endregion

        #region Evento de detección de cambios en la conexión a la red
        /// <summary>
        /// Este manejador de evento gestiona el evento que se dispara cuando hay un cambio en la conectividad a la red
        /// </summary>
        void NetworkChange_NetworkAddressChanged(object sender, EventArgs e)
        {
            bool isThereNetwork =
            Microsoft.Phone.Net.NetworkInformation.NetworkInterface.GetIsNetworkAvailable();

            if (isThereNetwork)
            {
                // Tenemos conectividad a Internet, por lo que lanzamos la transferencia de órdenes
                this.Transferir();
            }
        }
        #endregion

    }

}