using System;
using System.ComponentModel;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Runtime.Serialization;
using SGPM.PhonePER;

namespace SGPM.PhonePER.ModeloDatos
{
    [Database(Name = "SGPM")]
    public partial class SGPMDataContext : DataContext
    {
        private static MappingSource m_MappingSource = new AttributeMappingSource();

        #region Definiciones de mtodos de extensibilidad

        partial void OnCreated();
        partial void InsertOrdenTrabajo(OrdenTrabajo instance);
        partial void UpdateOrdenTrabajo(OrdenTrabajo instance);
        partial void DeleteOrdenTrabajo(OrdenTrabajo instance);
        partial void InsertPerito(Perito instance);
        partial void UpdatePerito(Perito instance);
        partial void DeletePerito(Perito instance);

        #endregion

        // Se enva la cadena de conexin a la clase base
        public SGPMDataContext() :
            base("Data Source=isostore:/SGPM.sdf", m_MappingSource)
        {
        }

        // Tabla para las rdenes de trabajo
        public Table<OrdenTrabajo> OrdenesTrabajo
        {
            get
            {
                return this.GetTable<OrdenTrabajo>();
            }
        }

        // Tabla para los peritos
        public Table<Perito> Peritos
        {
            get
            {
                return this.GetTable<Perito>();
            }
        }
    }

    [Table(Name = "OrdenesTrabajo")]
    public partial class OrdenTrabajo : INotifyPropertyChanging, INotifyPropertyChanged
    {
        private static PropertyChangingEventArgs m_EmptyChangingEventArgs = new PropertyChangingEventArgs(String.Empty);

        private int m_IdOrdenTrabajo;
        private int m_IdPerito;
        private string m_NPoliza;
        private Nullable<DateTime> m_FechaValoracion;
        private byte m_Aceptada;
        private bool m_Valoracion;
        private string m_Descripcion;
        private string m_TipoSiniestro;
        private string m_Cliente;
        private string m_Direccion;
        private EntityRef<Perito> m_Perito;

        #region Definiciones de mtodos de extensibilidad

        partial void OnLoaded();
        partial void OnValidate(ChangeAction action);
        partial void OnCreated();
        partial void OnIdOrdenTrabajoChanging(int value);
        partial void OnIdOrdenTrabajoChanged();
        partial void OnIdPeritoChanging(int value);
        partial void OnIdPeritoChanged();
        partial void OnNPolizaChanging(string value);
        partial void OnNPolizaChanged();
        partial void OnFechaValoracionChanging(Nullable<DateTime> value);
        partial void OnFechaValoracionChanged();
        partial void OnAceptadaChanging(byte value);
        partial void OnAceptadaChanged();
        partial void OnDescripcionChanging(string value);
        partial void OnDescripcionChanged();
        partial void OnTipoSiniestroChanging(string value);
        partial void OnTipoSiniestroChanged();
        partial void OnClienteChanging(string value);
        partial void OnClienteChanged();
        partial void OnDireccionChanging(string value);
        partial void OnDireccionChanged();

        #endregion

        public OrdenTrabajo()
        {
            this.Initialize();
        }

        [Column(Name = "IdOrdenTrabajo", Storage = "m_IdOrdenTrabajo", DbType = "int NOT NULL", IsPrimaryKey = true)]
        public int IdOrdenTrabajo
        {
            get
            {
                return this.m_IdOrdenTrabajo;
            }

            set
            {
                if ((this.m_IdOrdenTrabajo != value))
                {
                    this.OnIdOrdenTrabajoChanging(value);
                    this.SendPropertyChanging();
                    this.m_IdOrdenTrabajo = value;
                    this.SendPropertyChanged("IdOrdenTrabajo");
                    this.OnIdOrdenTrabajoChanged();
                }
            }
        }

        [Column(Name = "IdPerito", Storage = "m_IdPerito", DbType = "int NOT NULL")]
        public int IdPerito
        {
            get
            {
                return this.m_IdPerito;
            }

            set
            {
                if ((this.m_IdPerito != value))
                {
                    this.OnIdPeritoChanging(value);
                    this.SendPropertyChanging();
                    this.m_IdPerito = value;
                    this.SendPropertyChanged("IdPerito");
                    this.OnIdPeritoChanged();
                }
            }
        }

        [Column(Storage = "m_NPoliza", DbType = "NVarChar(10) NOT NULL", CanBeNull = false)]
        public string NPoliza
        {
            get
            {
                return this.m_NPoliza;
            }

            set
            {
                if ((this.m_NPoliza != value))
                {
                    this.OnNPolizaChanging(value);
                    this.SendPropertyChanging();
                    this.m_NPoliza = value;
                    this.SendPropertyChanged("NPoliza");
                    this.OnNPolizaChanged();
                }
            }
        }

        [Column(Storage = "m_FechaValoracion", DbType = "DateTime NULL")]
        public Nullable<DateTime> FechaValoracion
        {
            get
            {
                return this.m_FechaValoracion;
            }

            set
            {
                if ((this.m_FechaValoracion != value))
                {
                    this.OnFechaValoracionChanging(value);
                    this.SendPropertyChanging();
                    this.m_FechaValoracion = value;
                    this.SendPropertyChanged("FechaValoracion");
                    this.OnFechaValoracionChanged();
                }
            }
        }

        [Column(Storage = "m_Aceptada", DbType = "int NOT NULL", CanBeNull = false)]
        public byte Aceptada
        {
            get
            {
                return this.m_Aceptada;
            }

            set
            {
                if ((this.m_Aceptada != value))
                {
                    this.OnAceptadaChanging(value);
                    this.SendPropertyChanging();
                    this.m_Aceptada = value;
                    this.SendPropertyChanged("Aceptada");
                    this.OnAceptadaChanged();
                }
            }
        }

        public bool Valoracion
        {
            get
            {
                return this.m_Valoracion;
            }

            set
            {
                if ((this.m_Valoracion != value))
                {
                    this.m_Valoracion = value;
                }
            }
        }


        [Column(Storage = "m_Descripcion", DbType = "NVarChar(255) NULL", CanBeNull = true)]
        public string Descripcion
        {
            get
            {
                return this.m_Descripcion;
            }

            set
            {
                if ((this.m_Descripcion != value))
                {
                    this.OnDescripcionChanging(value);
                    this.SendPropertyChanging();
                    this.m_Descripcion = value;
                    this.SendPropertyChanged("Descripcion");
                    this.OnDescripcionChanged();
                }
            }
        }

        [Column(Storage = "m_TipoSiniestro", DbType = "NVarChar(100) NOT NULL", CanBeNull = false)]
        public string TipoSiniestro
        {
            get
            {
                return this.m_TipoSiniestro;
            }

            set
            {
                if ((this.m_TipoSiniestro != value))
                {
                    this.OnTipoSiniestroChanging(value);
                    this.SendPropertyChanging();
                    this.m_TipoSiniestro = value;
                    this.SendPropertyChanged("TipoSiniestro");
                    this.OnTipoSiniestroChanged();
                }
            }
        }

        [Column(Storage = "m_Cliente", DbType = "NVarChar(100) NOT NULL", CanBeNull = false)]
        public string Cliente
        {
            get
            {
                return this.m_Cliente;
            }

            set
            {
                if ((this.m_Cliente != value))
                {
                    this.OnClienteChanging(value);
                    this.SendPropertyChanging();
                    this.m_Cliente = value;
                    this.SendPropertyChanged("Cliente");
                    this.OnClienteChanged();
                }
            }
        }

        [Column(Storage = "m_Direccion", DbType = "NVarChar(255) NOT NULL", CanBeNull = false)]
        public string Direccion
        {
            get
            {
                return this.m_Direccion;
            }

            set
            {
                if ((this.m_Direccion != value))
                {
                    this.OnDireccionChanging(value);
                    this.SendPropertyChanging();
                    this.m_Direccion = value;
                    this.SendPropertyChanged("Direccion");
                    this.OnDireccionChanged();
                }
            }
        }

        // La columna Version ayuda a mejorar el rendimiento
        [Column(IsVersion = true)]
        private Binary _version;

        [Association(Name = "Perito_OrdenTrabajo", Storage = "m_Perito", ThisKey = "IdPerito", OtherKey = "IdPerito", IsForeignKey = true)]
        public Perito Perito
        {
            get
            {
                return this.m_Perito.Entity;
            }

            set
            {
                Perito previousValue = this.m_Perito.Entity;
                if (((previousValue != value)
                            || (this.m_Perito.HasLoadedOrAssignedValue == false)))
                {
                    this.SendPropertyChanging();
                    if ((previousValue != null))
                    {
                        this.m_Perito.Entity = null;
                        previousValue.OrdenesTrabajo.Remove(this);
                    }
                    this.m_Perito.Entity = value;
                    if ((value != null))
                    {
                        value.OrdenesTrabajo.Add(this);
                        this.m_IdPerito = value.IdPerito;
                    }
                    else
                    {
                        this.m_IdPerito = default(System.Int32);
                    }
                    this.SendPropertyChanged("Perito");
                }
            }
        }

        public event PropertyChangingEventHandler PropertyChanging;

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void SendPropertyChanging()
        {
            if ((this.PropertyChanging != null))
            {
                this.PropertyChanging(this, m_EmptyChangingEventArgs);
            }
        }

        protected virtual void SendPropertyChanged(String propertyName)
        {
            if ((this.PropertyChanged != null))
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private void Initialize()
        {
            this.m_Perito = default(EntityRef<Perito>);
            OnCreated();
        }

        [OnDeserializing]
        public void OnDeserializing(StreamingContext context)
        {
            this.Initialize();
        }

        public static implicit operator ProxyWSPhonePER.OrdenTrabajoPhone(OrdenTrabajo orden)
        {
            if (orden == null)
            {
                return null;
            }

            return new ProxyWSPhonePER.OrdenTrabajoPhone()
            {
                IdOrdenTrabajo = orden.IdOrdenTrabajo,
                IdPerito = orden.IdPerito,
                NPoliza = orden.NPoliza,
                FechaValoracion = orden.FechaValoracion,
                Aceptada = orden.Aceptada,
                Descripcion = orden.Descripcion,
                TipoSiniestro = orden.TipoSiniestro,
                Cliente = orden.Cliente,
                Direccion = orden.Direccion
            };
        }

        public static implicit operator OrdenTrabajo(ProxyWSPhonePER.OrdenTrabajoPhone orden)
        {
            if (orden == null)
            {
                return null;
            }

            return new OrdenTrabajo()
            {
                IdOrdenTrabajo = orden.IdOrdenTrabajo,
                IdPerito = (int)orden.IdPerito,
                NPoliza = orden.NPoliza,
                //FechaValoracion = orden.FechaValoracion,
                Aceptada = orden.Aceptada,
                Descripcion = orden.Descripcion,
                TipoSiniestro = orden.TipoSiniestro,
                Cliente = orden.Cliente,
                Direccion = orden.Direccion
            };
        }
    }

    [Table(Name = "Peritos")]
    public partial class Perito : INotifyPropertyChanging, INotifyPropertyChanged
    {
        private static PropertyChangingEventArgs m_EmptyChangingEventArgs = new PropertyChangingEventArgs(String.Empty);

        private int m_IdPerito;
        private string m_Nombre;
        private string m_Apellidos;
        private string m_Login;
        private string m_Password;
        private EntitySet<OrdenTrabajo> m_OrdenesTrabajo;
        private bool m_Serializing;

        #region Definiciones de mtodos de extensibilidad

        partial void OnLoaded();
        partial void OnValidate(ChangeAction action);
        partial void OnCreated();
        partial void OnIdPeritoChanging(int value);
        partial void OnIdPeritoChanged();
        partial void OnNombreChanging(string value);
        partial void OnNombreChanged();
        partial void OnApellidosChanging(string value);
        partial void OnApellidosChanged();
        partial void OnLoginChanging(string value);
        partial void OnLoginChanged();
        partial void OnPasswordChanging(string value);
        partial void OnPasswordChanged();

        #endregion

        public Perito()
        {
            this.Initialize();
        }

        [Column(Name = "IdPerito", Storage = "m_IdPerito", DbType = "int NOT NULL", IsPrimaryKey = true)]
        public int IdPerito
        {
            get
            {
                return this.m_IdPerito;
            }

            set
            {
                if ((this.m_IdPerito != value))
                {
                    this.OnIdPeritoChanging(value);
                    this.SendPropertyChanging();
                    this.m_IdPerito = value;
                    this.SendPropertyChanged("IdPerito");
                    this.OnIdPeritoChanged();
                }
            }
        }

        [Column(Storage = "m_Nombre", DbType = "NVarChar(25) NOT NULL", CanBeNull = false)]
        public string Nombre
        {
            get
            {
                return this.m_Nombre;
            }

            set
            {
                if ((this.m_Nombre != value))
                {
                    this.OnNombreChanging(value);
                    this.SendPropertyChanging();
                    this.m_Nombre = value;
                    this.SendPropertyChanged("Nombre");
                    this.OnNombreChanged();
                }
            }
        }

        [Column(Storage = "m_Apellidos", DbType = "NVarChar(50) NOT NULL", CanBeNull = false)]
        public string Apellidos
        {
            get
            {
                return this.m_Apellidos;
            }

            set
            {
                if ((this.m_Apellidos != value))
                {
                    this.OnApellidosChanging(value);
                    this.SendPropertyChanging();
                    this.m_Apellidos = value;
                    this.SendPropertyChanged("Apellidos");
                    this.OnApellidosChanged();
                }
            }
        }

        [Column(Storage = "m_Login", DbType = "NVarChar(8) NOT NULL", CanBeNull = false)]
        public string Login
        {
            get
            {
                return this.m_Login;
            }

            set
            {
                if ((this.m_Login != value))
                {
                    this.OnLoginChanging(value);
                    this.SendPropertyChanging();
                    this.m_Login = value;
                    this.SendPropertyChanged("Login");
                    this.OnLoginChanged();
                }
            }
        }

        [Column(Storage = "m_Password", DbType = "NVarChar(10) NOT NULL", CanBeNull = false)]
        public string Password
        {
            get
            {
                return this.m_Password;
            }

            set
            {
                if ((this.m_Password != value))
                {
                    this.OnPasswordChanging(value);
                    this.SendPropertyChanging();
                    this.m_Password = value;
                    this.SendPropertyChanged("Password");
                    this.OnPasswordChanged();
                }
            }
        }

        // La columna Version ayuda a mejorar el rendimiento
        [Column(IsVersion = true)]
        private Binary _version;

        [Association(Name = "Perito_OrdenTrabajo", Storage = "m_OrdenesTrabajo", ThisKey = "IdPerito", OtherKey = "IdPerito")]
        public EntitySet<OrdenTrabajo> OrdenesTrabajo
        {
            get
            {
                if ((this.m_Serializing && (this.m_OrdenesTrabajo.HasLoadedOrAssignedValues == false)))
                {
                    return null;
                }
                return this.m_OrdenesTrabajo;
            }

            set
            {
                this.m_OrdenesTrabajo.Assign(value);
            }
        }

        public event PropertyChangingEventHandler PropertyChanging;

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void SendPropertyChanging()
        {
            if ((this.PropertyChanging != null))
            {
                this.PropertyChanging(this, m_EmptyChangingEventArgs);
            }
        }

        protected virtual void SendPropertyChanged(String propertyName)
        {
            if ((this.PropertyChanged != null))
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private void AttachOrdenTrabajo(OrdenTrabajo entity)
        {
            this.SendPropertyChanging();
            entity.Perito = this;
        }

        private void DetachOrdenTrabajo(OrdenTrabajo entity)
        {
            this.SendPropertyChanging();
            entity.Perito = null;
        }

        private void Initialize()
        {
            this.m_OrdenesTrabajo = new EntitySet<OrdenTrabajo>(new Action<OrdenTrabajo>(this.AttachOrdenTrabajo), new Action<OrdenTrabajo>(this.DetachOrdenTrabajo));
            OnCreated();
        }

        [OnDeserializing]
        public void OnDeserializing(StreamingContext context)
        {
            this.Initialize();
        }

        [OnSerializing]
        public void OnSerializing(StreamingContext context)
        {
            this.m_Serializing = true;
        }

        [OnSerialized]
        public void OnSerialized(StreamingContext context)
        {
            this.m_Serializing = false;
        }

        public static implicit operator Perito(ProxyWSPhonePER.PeritoPhone usuario)
        {
            if (usuario == null)
            {
                return null;
            }

            return new Perito()
            {
                IdPerito = usuario.IdPerito,
                Nombre = usuario.Nombre,
                Apellidos = usuario.Apellidos,
                Login = usuario.Login,
                Password = usuario.Password
            };
        }

        public static implicit operator ProxyWSPhonePER.PeritoPhone(Perito usuario)
        {
            if (usuario == null)
            {
                return null;
            }

            return new ProxyWSPhonePER.PeritoPhone()
            {
                IdPerito = usuario.IdPerito,
                Nombre = usuario.Nombre,
                Apellidos = usuario.Apellidos,
                Login = usuario.Login,
                Password = usuario.Password
            };
        }
    }
}
