﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Text;
using SGPM.PhonePER.ModeloDatos;
using SGPM.PhonePER.Enums;
using SGPM.PhonePER.ProxyWSPhonePER;
using Microsoft.Phone.Controls;

using Microsoft.Phone.Shell;
using Microsoft.Phone.Tasks;
using Microsoft.Phone;
using System.IO;
using System.IO.IsolatedStorage;
using System.Windows.Controls.Primitives;
using System.Windows.Media.Imaging;

namespace SGPM.PhonePER
{
    /// <summary>
    /// Página para la valoración de una orden de trabajo 
    /// </summary>
    public partial class OrdenTrabajoPage : PhoneApplicationPage
    {
        #region Variables miembro
        // Instancia del servicio web
        WSPhonePERClient _ws = new WSPhonePERClient();

        //Id. de la orden de trabajo a valorar
        private int _IdOrdenTrabajo;

        //El selector de la cámara para capturar una imagen
        CameraCaptureTask ctask;

        //Objeto donde se almacena la captura de imagen
        private WriteableBitmap _CapturedImage;

        //Popup de ayuda
        private Popup _help = new Popup();

        #endregion

        #region Constructor

        public OrdenTrabajoPage()
        {
            InitializeComponent();

            //Crea una nueva instancia de CameraCaptureClass
            ctask = new CameraCaptureTask();

            //Crea un nuevo manejador de evento para capturar la foto
            ctask.Completed += new EventHandler<PhotoResult>(ctask_Completed);

        }

        #endregion

        #region Manejador de eventos de navegación

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);

            using (SGPMDataContext _DataContext = new SGPMDataContext())
            {
                // Obtenemos la órden de trabajo actual
                _IdOrdenTrabajo = Convert.ToInt32(this.NavigationContext.QueryString["IdOrdenTrabajo"]);
                var orden = _DataContext.OrdenesTrabajo.Single(o => o.IdOrdenTrabajo == _IdOrdenTrabajo);

                // Actualizamos el valor booleano de la valoración en función del estado
                orden.Valoracion = (orden.Aceptada == (int)ValoracionOrden.Aceptada ? true : false);

                // Asignamos el contexto de datos para el enlace con los controles
                this.DataContext = orden;
            }

        }

        #endregion

        #region Manejador de eventos de la botonera

        /// <summary>
        /// Manejador del evento click para el botón de la cámara
        /// Abre la cámara en el teléfono
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnImagen_Click(object sender, EventArgs e)
        {
            //Muestra la cámara
            ctask.Show();

        }

        /// <summary>
        /// Manejador del evento click para el botón de guardar
        /// Almacena en la base de datos local la orden ya valorada, para su posterior transferencia
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSave_Click(object sender, EventArgs e)
        {
            StringBuilder sb = new StringBuilder();

            // Control de avisos. Informamos al perito 
            if (!chkAceptada.IsChecked.Value)
            {
                sb.AppendLine("No ha chequeado la valoración.");
            }
            if (String.IsNullOrEmpty(tbxDescripcion.Text))
            {
                sb.AppendLine("No ha introducido ninguna observación.");
            }

            if (sb.Length > 0)
            {
                // Se informa al perito de los datos que pueden faltar
                sb.AppendLine("¿Desea continuar?");
                if (MessageBox.Show(sb.ToString(), "Aviso", MessageBoxButton.OKCancel) == MessageBoxResult.Cancel)
                {
                    //No almacenamos la orden
                    return;
                }
            }

            // Almacenamos la orden de trabajo en la base de datos del teléfono
            try
            {
                using (SGPMDataContext _DataContext = new SGPMDataContext())
                {
                    //Recupera la orden de trabajo
                    OrdenTrabajo existingObject = _DataContext.OrdenesTrabajo.Where(x => x.IdOrdenTrabajo == _IdOrdenTrabajo).FirstOrDefault();

                    existingObject.FechaValoracion = DateTime.Today;
                    if (chkAceptada.IsChecked.Value)
                    {
                        existingObject.Aceptada = (int)ValoracionOrden.Aceptada; // Aceptada
                    }
                    else
                    {
                        existingObject.Aceptada = (int)ValoracionOrden.Denegada; // Denegada
                    }
                    existingObject.Descripcion = tbxDescripcion.Text;
                    _DataContext.SubmitChanges();
                }
            }
            catch
            {
                MessageBox.Show("Error al almacenar la orden en el repositorio local.", "Error", MessageBoxButton.OK);
            }

            // Volvemos a la página principal
            this.NavigationService.GoBack();
        }

        /// <summary>
        /// Manejador de evento para el botón de ayuda
        /// Crea una ventana popup para mostrar el contenido de la ayuda
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void btnAyuda_Click(object sender, EventArgs e)
        {
            //Stack panel cn el fondo negro
            StackPanel panelHelp = new StackPanel();
            panelHelp.Background = new SolidColorBrush(Colors.Black);
            panelHelp.Width = 300;
            panelHelp.Height = 400;

            //Crea un borde blanco
            Border border = new Border();
            border.BorderBrush = new SolidColorBrush(Colors.White);
            border.BorderThickness = new Thickness(7.0);

            //Boton de cerrar para salir del popup
            Button close = new Button();
            close.Content = "Cerrar";
            close.Margin = new Thickness(5.0);
            close.Click += new RoutedEventHandler(close_Click);

            //Crea el texto de la ayuda
            TextBlock textblockHelp = new TextBlock();
            textblockHelp.FontSize = 24;
            textblockHelp.Foreground = new SolidColorBrush(Colors.White);
            textblockHelp.TextWrapping = TextWrapping.Wrap;
            textblockHelp.Text = "Pulse sobre el botón con imagen de cámara para tomar una foto.\n" +
                "Esto mostrará la herramienta para capturar fotos. Una vez hecha la puede aceptar o rechazar.\n";
            textblockHelp.Margin = new Thickness(5.0);

            //Añade los controles al stack panel
            panelHelp.Children.Add(textblockHelp);
            panelHelp.Children.Add(close);
            border.Child = panelHelp;

            // Inicializa la propiedad Child del popup
            _help.Child = border;

            // Localización en la pantalla de la ventana popup
            _help.VerticalOffset = 200;
            _help.HorizontalOffset = 85;

            // Se abre la ventana
            _help.IsOpen = true;
        }

        /// <summary>
        /// Manejador de evento para el botón cerrar de la ventana de ayuda
        /// Cierra el popup
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void close_Click(object sender, RoutedEventArgs e)
        {
            _help.IsOpen = false;
        }
        #endregion

        #region Manejador de eventos de la foto

        private void btnVerFoto_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            //Si el usuario no ha tomado una foto, no se puede visualizar nada
            if (_CapturedImage == null)
            {
                MessageBox.Show("Primero debe capturar una imagen.", "Error", MessageBoxButton.OK);
            }
            else
            {
                string parametros = string.Format("?IdOrdenTrabajo={0}", _IdOrdenTrabajo);

                //Si se ha tomado una foto, se navega a la pantalla que muestra la imagen
                this.NavigationService.Navigate(new Uri(string.Format("/VerFotoPage.xaml{0}", parametros),
                    UriKind.Relative));

            }

        }

        /// <summary>
        /// Manejador de evento para el borrado físico de la foto
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnEliminarFoto_Click(object sender, System.Windows.RoutedEventArgs e)
        {

            if (_CapturedImage != null)
            {

                //Crea un fichero JPEG en el almacenamiento aislado
                String tempJPEG = string.Format("TempJPEG_{0}.jpg", _IdOrdenTrabajo);

                //Crea el almacenamiento virtual y chequea si no existe otro fichero
                using (IsolatedStorageFile myStore = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    if (myStore.FileExists(tempJPEG))
                    {
                        myStore.DeleteFile(tempJPEG);
                    }
                }
                //Ocultamos el botón para visualizar la captura
                btnVerFoto.Visibility = Visibility.Collapsed;
                btnEliminarFoto.Visibility = Visibility.Collapsed;
            }
        }

        #endregion

        #region Manejador de eventos de captura de fotos

        /// <summary>
        /// Manejador de evento para capturar el stream de la imagen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ctask_Completed(object sender, PhotoResult e)
        {

            if (e.TaskResult == TaskResult.OK && e.ChosenPhoto != null)
            {

                //Toma el stream JPEG y lo decodifica a un objeto WriteableBitmap 
                _CapturedImage = PictureDecoder.DecodeJpeg(e.ChosenPhoto);

                //Crea un fichero JPEG en el almacenamiento aislado
                String tempJPEG = string.Format ("TempJPEG_{0}.jpg",_IdOrdenTrabajo);

                //Crea el almacenamiento virtual y chequea si no existe otro fichero
                var myStore = IsolatedStorageFile.GetUserStoreForApplication();
                if (myStore.FileExists(tempJPEG))
                {
                    myStore.DeleteFile(tempJPEG);
                }
                IsolatedStorageFileStream myFileStream = myStore.CreateFile(tempJPEG);

                //Codifica el WriteableBitmap en un stream JPEG y lo almacena en el almacenamiento aislado
                Extensions.SaveJpeg(_CapturedImage, myFileStream, _CapturedImage.PixelWidth, _CapturedImage.PixelHeight, 0, 85);
                
                myFileStream.Close();

                //Mostramos el botón para visualizar la captura
                btnVerFoto.Visibility = Visibility.Visible;
                btnEliminarFoto.Visibility = Visibility.Visible;
            }
        }

        #endregion

    }
}