﻿using System;
using System.Windows;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using SGPM.PhonePER.ModeloDatos;
using SGPM.PhonePER.ProxyWSPhonePER;

namespace SGPM.PhonePER
{
    public partial class UserPassPage : PhoneApplicationPage
    {

        private AppSettings settings = new AppSettings();

        public UserPassPage()
        {
            // Asignación de controladores de eventos para la página
            // Indicamos que al cargar la página queremos manejar el evento
            this.Loaded += new RoutedEventHandler(UserPassPage_Loaded);

            InitializeComponent();
        }

        private void UserPassPage_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                // No se ha iniciado la sesión
                App.Autenticado = false;

                using (SGPMDataContext _DataContext = new SGPMDataContext())
                {

                    if (!_DataContext.DatabaseExists())
                    {
                        // No existe, se crea la base de datos
                        _DataContext.CreateDatabase();
                    }
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex);
                MessageBox.Show("Ha ocurrido un error al inicializar la aplicación. La aplicación se cerrará.", "Error", MessageBoxButton.OK);

                // Volvemos a la página principal
                this.NavigationService.GoBack();
            }

        }

        /// <summary>
        /// Manejador de eventos del botón Aceptar
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void doneButton_Click(object sender, EventArgs e)
        {
            // La página queda congelada
            this.IsEnabled = false;
            this.ApplicationBar.IsMenuEnabled = false;

            try
            {
                // Instanciamos al servicio web para la validación
                WSPhonePERClient ws = new WSPhonePERClient();

                // Se asigna el manejador de eventos
                ws.ValidaUsuarioCompleted +=
                    new EventHandler<ValidaUsuarioCompletedEventArgs>(ws_ValidaUsuarioCompleted);
                
                // Invoca la validación del perito
                ws.ValidaUsuarioAsync(textBoxUsername.Text, passwordBoxPassword.Password);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex);
                MessageBox.Show("Ha ocurrido un error al validar al usuario.", "Error", MessageBoxButton.OK);
            }

        }

        /// <summary>
        /// Manejador de evento del botón Cancelar
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void cancelButton_Click(object sender, EventArgs e)
        {
            textBoxUsername.Text="";
            passwordBoxPassword.Password = "";
            chkRememberMe.IsChecked = false;
        }

        private void ws_ValidaUsuarioCompleted(object sender, ValidaUsuarioCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show("No se ha podido autenticar en la aplicación. Realice otro intento.", "Error", MessageBoxButton.OK);
            }
            else
            {
                // Perito autenticado
                var perito = e.Result;

                if (perito != null)
                {
                    try
                    {
                        if (this.chkRememberMe.IsChecked == true)
                        {
                            // Almacenamos el usuario y password para la próxima vez que acceda
                            settings.UsernameSetting = textBoxUsername.Text;
                            settings.PasswordSetting = passwordBoxPassword.Password;
                            settings.RememberMeSetting = true;
                        }
                        else
                        {
                            settings.RememberMeSetting = false;
                        }

                        MessageBox.Show("Autenticado", "Acceso al sistema PhonePER", MessageBoxButton.OK);

                        using (SGPMDataContext _DataContext = new SGPMDataContext())
                        {
                            // Comprobamos si el último usuario que utilizó la aplicación es el mismo que acaba de iniciar sesión
                            // Si el último perito en utilizar la aplicación es el mismo que el actual, para 
                            // borrar y volver a crear la base de datos
                            if (!_DataContext.Peritos.Any(p => p.IdPerito == perito.IdPerito))
                            {
                                // Peritos distintos, se tiene que volver a inicializar la base de datos
                                _DataContext.DeleteDatabase();
                                _DataContext.CreateDatabase();

                                // Se inserta el perito
                                _DataContext.Peritos.InsertOnSubmit(perito);
                                _DataContext.SubmitChanges();
                            }

                        }

                        // Se ha autenticado en la aplicación
                        App.Autenticado = true;
                    }
                    catch (Exception ex)
                    {
                        System.Diagnostics.Debug.WriteLine(ex);
                        MessageBox.Show("Ha ocurrido un error al intentar acceder a la base de datos. La aplicación se cerrará.", "Error", MessageBoxButton.OK);
                    }

                    // Vamos a la página del listado de órdenes pendientes
                    this.NavigationService.Navigate(new Uri("/ListOrdenesPage.xaml",UriKind.Relative));
                }
                else
                {
                    MessageBox.Show("Usuario o password incorrectos. Revise los datos.", "Sesión no iniciada", MessageBoxButton.OK);
                }
            }

            // Habilitamos la página
            this.IsEnabled = true;
            this.ApplicationBar.IsMenuEnabled = true;
        }
    }
}