﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Web;
using System.Web.Mvc;
using SGPM.WebPER.ProxyWSWebPER;
using SGPM.CrossCutting.Logging;
using SGPM.CrossCutting.Enums;
using SGPM.WebPER.Utils;

namespace SGPM.WebPER.Controllers
{
    /// <summary>
    /// Este controlador maneja todas las acciones relacionadas con los informes
    /// </summary>
    public class InformeController : RepositoryBasedController
    {

        #region Acciones
        ///
        /// GET: /Informe/
        /// <summary>
        /// Listado de informes
        /// </summary>
        [SessionExpireFilter]
        public ActionResult Index()
        {
            // Carga de los Filtros de los desplegables
            // Tipos de Informes
            List<SelectListItem> tipoInformes = new List<SelectListItem>();
            tipoInformes.Add(new SelectListItem { Text = "Ratio de siniestralidad", Value = "1" });
            tipoInformes.Add(new SelectListItem { Text = "Ratio de productividad", Value = "2" });

            ViewBag.TipoInformes = tipoInformes.ToList();

            try
            {

                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    // Años
                    List<SelectListItem> annos = new List<SelectListItem>();
                    annos.AddRange(_ws.ListAnnos(this._gestor).Select(c => new SelectListItem
                    {
                        Text = c.Value.ToString(),
                        Value = c.Value.ToString()
                    }).ToList());
                    ViewBag.Annos = annos;
                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                // La sesión ha expirado
                return RedirectToAction("LogOn", "Account");
            }
            return View();
        }

        /// <summary>
        /// Solicita un informe de ratios 
        /// </summary>
        /// <param name="anno">Año por el que se filtra el informe.</param>
        /// <param name="tipoInforme">Indica el tipo de informe que desea.</param>
        /// <returns>Un objeto Json con el informe.</returns>
        [HttpPost]
        public JsonResult InformeList(int anno, int tipoInforme, int jtStartIndex = 0,
            int jtPageSize = 0, string jtSorting = null)
        {
            try
            {
                using (ProxyWSWebPER.WSWebPERClient _ws = new ProxyWSWebPER.WSWebPERClient())
                {
                    // Evaluamos el tipo de informe solicitado
                    if (tipoInforme == 1)
                    {
                        // Ratio de siniestralidad
                        var informeCount = _ws.GetRatioSiniestralidadCountByFilter(anno);

                        if (informeCount < 0)
                        {
                            //log
                            return Json(new { Result = "ERROR", Message = "No se ha podido obtener el conteo de elementos." });
                        }

                        var informe = _ws.ListRatioSiniestralidad(this._gestor, anno, jtStartIndex, jtPageSize, jtSorting);

                        return Json(new { Result = "OK", Records = informe, TotalRecordCount = informeCount });
                    }
                    else
                    {
                        // Ratio de productividad
                        var informeCount = _ws.GetRatioProductividadCountByFilter(anno);

                        if (informeCount < 0)
                        {
                            //log
                            return Json(new { Result = "ERROR", Message = "No se ha podido obtener el conteo de elementos." });
                        }

                        var informe = _ws.ListRatioProductividad(this._gestor, anno, jtStartIndex, jtPageSize, jtSorting);

                        return Json(new { Result = "OK", Records = informe, TotalRecordCount = informeCount });
                    }

                }
            }
            catch (Exception ex)
            {
                Tracer.Logger.Error(ex);
                return Json(new { Result = "ERROR", Message = ex.Message });
            }
        }


        #endregion

    }
}
