﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using System.Reflection;

namespace SGPM.WebPER.Utils
{

    /// <summary>
    /// Esta clase implementa un filtro de acción, que se emplea en MVC en los controladores para detectar que la sesión ha expirado
    /// De esta forma se redirige la ejecución a la página de autenticación.
    /// </summary>
    public class SessionExpireFilterAttribute : ActionFilterAttribute
    {

        public override void OnActionExecuting(ActionExecutingContext filterContext)
        {
            HttpContext ctx = HttpContext.Current;

            // Comprobamos si existe la sesión
            if (ctx.Session != null)
            {

                // Comprobamos si se ha generado una nueva sesión
                if (ctx.Session.IsNewSession)
                {

                    // Si es una nueva sesión pero existe una cookie de sesión, entonces
                    // se trata de un timeout
                    string sessionCookie = ctx.Request.Headers["Cookie"];
                    if ((null != sessionCookie) && (sessionCookie.IndexOf("ASP.NET_SessionId") >= 0))
                    {
                        // Redirigimos a la página de logoff para limpirar la sesión y posteriormete
                        // volver a autenticarse.
                        ctx.Response.Redirect("~/Account/LogOff");
                    }
                }
            }

            base.OnActionExecuting(filterContext);
        }
    }
}
