﻿using System.Collections.Generic;
using System.ServiceModel;
using SGPM.WebServices;
using SGPM.WebServices.ModeloDatos;

namespace SGPM.WebServices
{
    /// <summary>
    /// Interfaz del servicio web WSPhonePER destinada a la aplicación móvil
    /// </summary>
    [ServiceContract]
    public interface IWSPhonePER
    {

        #region Usuarios
        /// <summary>
        /// Realiza el control de acceso a la aplicación de un usuario Perito
        /// </summary>
        /// <param name="login">Nombre de usuario del perito</param>
        /// <param name="password">Contraseña del perito</param>
        /// <returns>El perito que ha accedido correctamente a la aplicación</returns>
        [OperationContract]
        PeritoPhone ValidaUsuario(string login, string password);

        /// <summary>
        /// Realiza el cambio de contraseña de un usuario Perito
        /// </summary>
        /// <param name="perito">Usuario perito del que se quiere cambiar la constraseña</param>
        /// <param name="password">La contraseña a asignar al perito</param>
        /// <returns>True en caso que la operación se ha realizado con éxito, False en caso contrario.</returns>
        [OperationContract]
        bool CambiarPassword(PeritoPhone perito, string password);

        #endregion

        #region Ordenes de trabajo

        /// <summary>
        /// Devuelve un listado con todas las órdenes de trabajo asignadas a un perito y que están pendientes de recibir su valoración 
        /// </summary>
        /// <param name="perito">Usuario perito que solicita la operación</param>
        /// <returns>Un listado con todas las órdenes de trabajo asignadas al perito y que todavía no se han valorado</returns>
        [OperationContract]
        List<OrdenTrabajoPhone> ListOrdenes(PeritoPhone perito);

        /// <summary>
        /// Actualiza una orden de trabajo con la valoración del perito. La orden pasa a estado Cerrada
        /// </summary>
        /// <param name="perito">Usuario perito que solicita la operación</param>
        /// <param name="orden">Orden de trabajo que se ha valorado</param>
        /// <param name="pictureStream">Array de bytes que contiene la foto anexa</param>
        /// <returns>True en caso que la operación se haya efecturado correctamente, False en caso contratio</returns>
        [OperationContract]
        bool UpdateOrden(PeritoPhone perito, OrdenTrabajoPhone orden, byte[] pictureStream);

        #endregion

        #region Notificaciones Toast
        /// <summary>
        /// Recibe la URI que el Servicio de Notificaciones Push de Microsoft devuelve al cliente push cuando se
        /// crea el canal de notificaciones. De esta forma se vincula la URI de subscripción al dispositivo del perito.
        /// </summary>
        /// <param name="perito">Usuario perito al que se le vincula la URI</param>
        /// <param name="subscriptionURI">URI de subscripción al servicio de notificaciones push</param>
        /// <returns>Devuelve true si ha sido posible recepcionar la URI, false en caso contrario</returns>
        [OperationContract]
        bool SetSubscriptionURI(PeritoPhone perito, string subscriptionURI);
        
        #endregion
    }

}
